@@ -38,6 +38,8 @@ private:
   char newTimeshiftBufferDir[PATH_MAX];
   int newPlayRecording;
   int newAvoidEPGScan;
+  int newDisableScrambleTimeout;
+  int newDisableCamBlacklist;
 protected:
   virtual void Store(void);
 public:
