@@ -40,32 +40,25 @@ class cVideoBufferSimple : public cVideoBuffer
 {
 friend class cVideoBuffer;
 public:
-  virtual void Put(uint8_t *buf, unsigned int size);
+  virtual void Put(const uint8_t *buf, unsigned int size);
   virtual int ReadBlock(uint8_t **buf, unsigned int size, time_t &endTime, time_t &wrapTime);
 
 protected:
   cVideoBufferSimple();
-  virtual ~cVideoBufferSimple();
-  cRingBufferLinear *m_Buffer;
+  cRingBufferLinear m_Buffer;
   int m_BytesConsumed;
 };
 
 cVideoBufferSimple::cVideoBufferSimple()
+  :m_Buffer(MEGABYTE(5), TS_SIZE * 2, false)
 {
-  m_Buffer = new cRingBufferLinear(MEGABYTE(3), TS_SIZE * 2, false);
-  m_Buffer->SetTimeouts(0, 100);
+  m_Buffer.SetTimeouts(0, 100);
   m_BytesConsumed = 0;
 }
 
-cVideoBufferSimple::~cVideoBufferSimple()
+void cVideoBufferSimple::Put(const uint8_t *buf, unsigned int size)
 {
-  if (m_Buffer)
-    delete m_Buffer;
-}
-
-void cVideoBufferSimple::Put(uint8_t *buf, unsigned int size)
-{
-  m_Buffer->Put(buf, size);
+  m_Buffer.Put(buf, size);
 }
 
 int cVideoBufferSimple::ReadBlock(uint8_t **buf, unsigned int size, time_t &endTime, time_t &wrapTime)
@@ -73,10 +66,10 @@ int cVideoBufferSimple::ReadBlock(uint8_t **buf, unsigned int size, time_t &endT
   int  readBytes;
   if (m_BytesConsumed)
   {
-    m_Buffer->Del(m_BytesConsumed);
+    m_Buffer.Del(m_BytesConsumed);
   }
   m_BytesConsumed = 0;
-  *buf = m_Buffer->Get(readBytes);
+  *buf = m_Buffer.Get(readBytes);
   if (!(*buf) || readBytes < TS_SIZE)
   {
     usleep(100);
@@ -94,7 +87,7 @@ int cVideoBufferSimple::ReadBlock(uint8_t **buf, unsigned int size, time_t &endT
 
   if ((*buf)[0] != TS_SYNC_BYTE)
   {
-    m_Buffer->Del(m_BytesConsumed);
+    m_Buffer.Del(m_BytesConsumed);
     m_BytesConsumed = 0;
     return 0;
   }
@@ -198,7 +191,7 @@ class cVideoBufferRAM : public cVideoBufferTimeshift
 {
 friend class cVideoBuffer;
 public:
-  virtual void Put(uint8_t *buf, unsigned int size);
+  virtual void Put(const uint8_t *buf, unsigned int size);
   virtual int ReadBlock(uint8_t **buf, unsigned int size, time_t &endTime, time_t &wrapTime);
   virtual void SetPos(off_t pos);
 
@@ -217,8 +210,7 @@ cVideoBufferRAM::cVideoBufferRAM()
 
 cVideoBufferRAM::~cVideoBufferRAM()
 {
-  if (m_Buffer)
-    free(m_Buffer);
+  free(m_Buffer);
 }
 
 bool cVideoBufferRAM::Init()
@@ -243,7 +235,7 @@ void cVideoBufferRAM::SetPos(off_t pos)
   m_BytesConsumed = 0;
 }
 
-void cVideoBufferRAM::Put(uint8_t *buf, unsigned int size)
+void cVideoBufferRAM::Put(const uint8_t *buf, unsigned int size)
 {
   if (Available() + MARGIN >= m_BufferSize)
   {
@@ -303,7 +295,7 @@ int cVideoBufferRAM::ReadBlock(uint8_t **buf, unsigned int size, time_t &endTime
   if (m_ReadPtr > (m_BufferSize - m_Margin))
   {
     int bytesToCopy = m_BufferSize - m_ReadPtr;
-    memmove(m_Buffer + (m_Margin - bytesToCopy), m_Buffer + m_ReadPtr, bytesToCopy);
+    memmove(m_Buffer + (m_Margin - bytesToCopy), m_BufferPtr + m_ReadPtr, bytesToCopy);
     *buf = m_Buffer + (m_Margin - bytesToCopy);
   }
   else
@@ -335,7 +327,7 @@ class cVideoBufferFile : public cVideoBufferTimeshift
 friend class cVideoBuffer;
 public:
   virtual off_t GetPosMax();
-  virtual void Put(uint8_t *buf, unsigned int size);
+  virtual void Put(const uint8_t *buf, unsigned int size);
   virtual int ReadBlock(uint8_t **buf, unsigned int size, time_t &endTime, time_t &wrapTime);
   virtual void SetPos(off_t pos);
 
@@ -393,10 +385,7 @@ bool cVideoBufferFile::Init()
   struct stat sb;
   if ((*TimeshiftBufferDir) && stat(TimeshiftBufferDir, &sb) == 0 && S_ISDIR(sb.st_mode))
   {
-    if (TimeshiftBufferDir[strlen(TimeshiftBufferDir)-1] == '/')
-      m_Filename = cString::sprintf("%sTimeshift-%d.vnsi", TimeshiftBufferDir, m_ClientID);
-    else
-      m_Filename = cString::sprintf("%s/Timeshift-%d.vnsi", TimeshiftBufferDir, m_ClientID);
+    m_Filename = cString::sprintf("%s/Timeshift-%d.vnsi", TimeshiftBufferDir, m_ClientID);
   }
   else
 #if VDRVERSNUM >= 20102
@@ -451,7 +440,7 @@ off_t cVideoBufferFile::GetPosMax()
   return posMax;
 }
 
-void cVideoBufferFile::Put(uint8_t *buf, unsigned int size)
+void cVideoBufferFile::Put(const uint8_t *buf, unsigned int size)
 {
   if (Available() + MARGIN >= m_BufferSize)
   {
@@ -626,22 +615,22 @@ class cVideoBufferRecording : public cVideoBufferFile
 friend class cVideoBuffer;
 public:
   virtual off_t GetPosMax();
-  virtual void Put(uint8_t *buf, unsigned int size);
+  virtual void Put(const uint8_t *buf, unsigned int size);
   virtual int ReadBlock(uint8_t **buf, unsigned int size, time_t &endTime, time_t &wrapTime);
   virtual time_t GetRefTime();
 
 protected:
-  cVideoBufferRecording(cRecording *rec);
+  cVideoBufferRecording(const cRecording *rec);
   virtual ~cVideoBufferRecording();
   virtual bool Init();
   virtual off_t Available();
   off_t GetPosEnd();
   cRecPlayer *m_RecPlayer;
-  cRecording *m_Recording;
+  const cRecording *m_Recording;
   cTimeMs m_ScanTimer;
 };
 
-cVideoBufferRecording::cVideoBufferRecording(cRecording *rec)
+cVideoBufferRecording::cVideoBufferRecording(const cRecording *rec)
 {
   m_Recording = rec;
   m_ReadCacheSize = 0;
@@ -662,7 +651,7 @@ off_t cVideoBufferRecording::GetPosMax()
   return cVideoBufferFile::GetPosMax();
 }
 
-void cVideoBufferRecording::Put(uint8_t *buf, unsigned int size)
+void cVideoBufferRecording::Put(const uint8_t *buf, unsigned int size)
 {
 
 }
@@ -778,7 +767,7 @@ class cVideoBufferTest : public cVideoBufferFile
 friend class cVideoBuffer;
 public:
   virtual off_t GetPosMax();
-  virtual void Put(uint8_t *buf, unsigned int size);
+  virtual void Put(const uint8_t *buf, unsigned int size);
 
 protected:
   cVideoBufferTest(cString filename);
@@ -818,7 +807,7 @@ off_t cVideoBufferTest::GetPosEnd()
   return end;
 }
 
-void cVideoBufferTest::Put(uint8_t *buf, unsigned int size)
+void cVideoBufferTest::Put(const uint8_t *buf, unsigned int size)
 {
 
 }
@@ -915,7 +904,7 @@ cVideoBuffer* cVideoBuffer::Create(cString filename)
     return buffer;
 }
 
-cVideoBuffer* cVideoBuffer::Create(cRecording *rec)
+cVideoBuffer* cVideoBuffer::Create(const cRecording *rec)
 {
   INFOLOG("Open recording: %s", rec->FileName());
   cVideoBufferRecording *buffer = new cVideoBufferRecording(rec);
